@@ -225,7 +225,7 @@ class Signal:
             # TODO: put this into the frame!
             self.frame.signals.append(self)
 
-        self.enumeration_format_re = {'re': '^\[(\d+)\]',
+        self.enumeration_format_re = {'re': r'^\[(\d+)\]',
                                       'format': '[{v}] {s}',
                                       'no_value_format': '{s}'}
 
@@ -735,7 +735,7 @@ class Frame(QtCanListener):
         unpacked = False
 
         if (msg.arbitration_id == self.id and
-                bool(msg.id_type) == self.extended):
+                bool(msg.is_extended_id) == self.extended):
             if self.mux_frame is None:
                 self.unpack(msg.data)
                 unpacked = True
